@@ -7,18 +7,21 @@ import hildon
 from os import listdir, path, system
 from ConfigParser import SafeConfigParser
 
-version = "1.6-0"
+version = "1.6-1"
 trackingEnabled = True
 stopTracking = True
 stopWhenIdle = False
 stopAfterMinutes = "5 minutes"
 confirmDisconnect = True
+showRestartButton = False
 showConnected = True
 connectedIcon = "Symbol"
 skipSystemCheck = False
 autostart = False
 keepWifiPMOff = False
-refreshPMAfterSec = "20 seconds"
+toggleWifiPM = False
+wifiPMOffSec = "2 seconds"
+wifiPMOnSec = "4 seconds"
 
 class Main(object):
     def __init__(self):
@@ -29,10 +32,13 @@ class Main(object):
         global showConnected
         global connectedIcon
         global confirmDisconnect
+        global showRestartButton
         global skipSystemCheck
         global autostart
         global keepWifiPMOff
-        global refreshPMAfterSec
+        global toggleWifiPM
+        global wifiPMOffSec
+        global wifiPMOnSec
         
         autostartDisabled = False
         keepWifiPMOffInstalled = False
@@ -56,6 +62,7 @@ class Main(object):
             self.parser.set('options', 'stopWhenIdle', str(stopWhenIdle))
             self.parser.set('options', 'stopAfterMinutes', str(connectedIcon))
             self.parser.set('options', 'confirmDisconnect', str(confirmDisconnect))
+            self.parser.set('options', 'showRestartButton', str(confirmDisconnect))
             self.parser.set('options', 'showConnected', str(showConnected))
             self.parser.set('options', 'connectedIcon', str(connectedIcon))
             self.parser.set('options', 'skipSystemCheck', str(skipSystemCheck))
@@ -84,6 +91,10 @@ class Main(object):
                     confirmDisconnect = self.parser.getboolean('options', 'confirmDisconnect')
                 else:
                     self.parser.set('options', 'confirmDisconnect', str(confirmDisconnect))
+                if self.parser.has_option('options', 'showRestartButton'):
+                    showRestartButton = self.parser.getboolean('options', 'showRestartButton')
+                else:
+                    self.parser.set('options', 'showRestartButton', str(showRestartButton))
                 if self.parser.has_option('options', 'showConnected'):
                     showConnected = self.parser.getboolean('options', 'showConnected')
                 else:
@@ -137,8 +148,17 @@ class Main(object):
         lines = sshFile.readlines()
         sshFile.close()
         for line in lines:
-            if 'sleep' in line:
-                refreshPMAfterSec = line.strip('\n').split()[1] + " seconds"
+            if line.strip('\n').strip(' ') == "iwconfig wlan0 power on #toggle":
+                toggleWifiPM = True
+            if "pmoff" in line:
+                wifiPMOffSec = line.strip('\n').strip(' ').split(' ')[1]
+                if wifiPMOffSec == "1":
+                    wifiPMOffSec += " second"
+                else:
+                    wifiPMOffSec += " seconds"
+            if "pmon" in line:
+                wifiPMOnSec = line.strip('\n').strip(' ').split(' ')[1] + " seconds"
+                break
         
         self.indicatorList = []
         for f in listdir('/usr/share/icons/hicolor/18x18/hildon/'):
@@ -149,7 +169,8 @@ class Main(object):
         self.indicatorList.insert(0, "Symbol")
         
         self.timeoutList = ["1 minute", "5 minutes", "15 minutes", "30 minutes", "60 minutes"]
-        self.pmRefreshList = ["5 seconds", "10 seconds", "15 seconds", "20 seconds"]
+        self.pmOffList = ["1 second", "2 seconds", "3 seconds"]
+        self.pmOnList = ["3 seconds", "4 seconds", "5 seconds"]
         
         self.main()
     
@@ -182,6 +203,10 @@ class Main(object):
         self.buttonConfirm = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonConfirm.set_label("Confirm when disconnecting")
         
+        self.buttonShowRestart = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonShowRestart.set_label("Show Restart button")
+        self.buttonShowRestart.set_alignment(0.08, 0.5)
+    
         self.buttonShowConnected = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonShowConnected.set_label("Show status 'connected' in status area")
     
@@ -200,14 +225,27 @@ class Main(object):
         self.buttonKeepWifiPMOff = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonKeepWifiPMOff.set_label("Keep wifi power management off when connected")
     
-        self.buttonRefreshPMAfterSec = hildon.PickerButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
-        self.buttonRefreshPMAfterSec.set_title("Refresh time:")
-        self.buttonRefreshPMAfterSec.set_alignment(0.1, 0.5, 0, 0)
-        self.buttonRefreshPMAfterSec.set_selector(self.selectPMRefreshTime())
+        self.buttonToggleWifiPM = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonToggleWifiPM.set_label("Toggle mode (less power consuming)")
+        self.buttonToggleWifiPM.set_alignment(0.12, 0.5)
+    
+        self.buttonWifiPMOffSec = hildon.PickerButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
+        self.buttonWifiPMOffSec.set_title("Wifi PM off:")
+        self.buttonWifiPMOffSec.set_alignment(0.17, 0.5, 0, 0)
+        self.buttonWifiPMOffSec.set_selector(self.selectWifiPMOffSec())
         try:
-            self.buttonRefreshPMAfterSec.set_active(self.pmRefreshList.index(refreshPMAfterSec))
+            self.buttonWifiPMOffSec.set_active(self.pmOffList.index(wifiPMOffSec))
         except:
-            self.buttonRefreshPMAfterSec.set_active(3)
+            self.buttonWifiPMOffSec.set_active(1)
+    
+        self.buttonWifiPMOnSec = hildon.PickerButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
+        self.buttonWifiPMOnSec.set_title("Wifi PM on:")
+        self.buttonWifiPMOnSec.set_alignment(0.17, 0.5, 0, 0)
+        self.buttonWifiPMOnSec.set_selector(self.selectWifiPMOnSec())
+        try:
+            self.buttonWifiPMOnSec.set_active(self.pmOnList.index(wifiPMOnSec))
+        except:
+            self.buttonWifiPMOnSec.set_active(1)
     
         vboxBorder = gtk.VBox(False, 0)
         vboxBorder.set_border_width(8)
@@ -219,12 +257,15 @@ class Main(object):
         vbox.pack_start(self.buttonStopWhenIdle, False, False, 0)
         vbox.pack_start(self.buttonStopAfterMinutes, False, False, 0)
         vbox.pack_start(self.buttonConfirm, False, False, 0)
+        vbox.pack_start(self.buttonShowRestart, False, False, 0)
         vbox.pack_start(self.buttonShowConnected, False, False, 0)
         vbox.pack_start(self.buttonConnectedIcon, False, False, 0)
         vbox.pack_start(labelOther, False, False, 8)
         vbox.pack_start(self.buttonAutostart, False, False, 0)
         vbox.pack_start(self.buttonKeepWifiPMOff, False, False, 0)
-        vbox.pack_start(self.buttonRefreshPMAfterSec, False, False, 0)
+        vbox.pack_start(self.buttonToggleWifiPM, False, False, 0)
+        vbox.pack_start(self.buttonWifiPMOffSec, False, False, 0)
+        vbox.pack_start(self.buttonWifiPMOnSec, False, False, 0)
         
         areaScrolling = hildon.PannableArea()
         areaScrolling.add_with_viewport(vbox)
@@ -238,16 +279,26 @@ class Main(object):
             self.buttonStopTracking.set_active(True)
         else:
             self.buttonStopTracking.set_active(False)
+
         if stopWhenIdle:
             self.buttonStopWhenIdle.set_active(True)
             self.buttonStopAfterMinutes.set_sensitive(True)
         else:
             self.buttonStopWhenIdle.set_active(False)
             self.buttonStopAfterMinutes.set_sensitive(False)
+
         if confirmDisconnect:
             self.buttonConfirm.set_active(True)
+            self.buttonShowRestart.set_sensitive(True)
         else:
             self.buttonConfirm.set_active(False)
+            self.buttonShowRestart.set_sensitive(False)
+
+        if showRestartButton:
+            self.buttonShowRestart.set_active(True)
+        else:
+            self.buttonShowRestart.set_active(False)
+
         if showConnected:
             self.buttonShowConnected.set_active(True)
             self.buttonConnectedIcon.set_sensitive(True)
@@ -263,6 +314,7 @@ class Main(object):
             self.buttonStopWhenIdle.set_sensitive(False)
             self.buttonStopAfterMinutes.set_sensitive(False)
             self.buttonConfirm.set_sensitive(False)
+            self.buttonShowRestart.set_sensitive(True)
             self.buttonShowConnected.set_sensitive(False)
             self.buttonConnectedIcon.set_sensitive(False)
         
@@ -270,23 +322,36 @@ class Main(object):
             self.buttonAutostart.set_active(True)
         else:
             self.buttonAutostart.set_active(False)
+
         if keepWifiPMOff:
             self.buttonKeepWifiPMOff.set_active(True)
-            self.buttonRefreshPMAfterSec.set_sensitive(True)
+            self.buttonToggleWifiPM.set_sensitive(True)
+            self.buttonWifiPMOffSec.set_sensitive(True)
+            self.buttonWifiPMOnSec.set_sensitive(True)
         else:
             self.buttonKeepWifiPMOff.set_active(False)
-            self.buttonRefreshPMAfterSec.set_sensitive(False)
+            self.buttonToggleWifiPM.set_sensitive(False)
+            self.buttonWifiPMOffSec.set_sensitive(False)
+            self.buttonWifiPMOnSec.set_sensitive(False)
         
+        if toggleWifiPM:
+            self.buttonToggleWifiPM.set_active(True)
+        else:
+            self.buttonToggleWifiPM.set_active(False)
+
         self.buttonTrackingEnabled.connect("clicked", self.buttonTrackingEnabled_clicked)
         self.buttonStopTracking.connect("clicked", self.buttonStopTracking_clicked)
         self.buttonStopWhenIdle.connect("clicked", self.buttonStopWhenIdle_clicked)
         self.buttonStopAfterMinutes.connect("value-changed", self.buttonStopAfterMinutes_changed)
         self.buttonConfirm.connect("clicked", self.buttonConfirm_clicked)
+        self.buttonShowRestart.connect("clicked", self.buttonShowRestart_clicked)
         self.buttonShowConnected.connect("clicked", self.buttonShowConnected_clicked)
         self.buttonConnectedIcon.connect("value-changed", self.buttonConnectedIcon_changed)
         self.buttonAutostart.connect("clicked", self.buttonAutostart_clicked)
         self.buttonKeepWifiPMOff.connect("clicked", self.buttonKeepWifiPMOff_clicked)
-        self.buttonRefreshPMAfterSec.connect("value-changed", self.buttonRefreshPMAfterSec_changed)
+        self.buttonToggleWifiPM.connect("clicked", self.buttonToggleWifiPM_clicked)
+        self.buttonWifiPMOffSec.connect("value-changed", self.buttonWifiPMOffSec_changed)
+        self.buttonWifiPMOnSec.connect("value-changed", self.buttonWifiPMOnSec_changed)
         
         menu = self.create_menu()
         window.set_app_menu(menu)
@@ -304,6 +369,10 @@ class Main(object):
             if stopWhenIdle:
                 self.buttonStopAfterMinutes.set_sensitive(True)
             self.buttonConfirm.set_sensitive(True)
+            if confirmDisconnect:
+                self.buttonShowRestart.set_sensitive(True)
+            else:
+                self.buttonShowRestart.set_sensitive(False)
             self.buttonShowConnected.set_sensitive(True)
             if showConnected:
                 self.buttonConnectedIcon.set_sensitive(True)
@@ -312,6 +381,7 @@ class Main(object):
             self.buttonStopWhenIdle.set_sensitive(False)
             self.buttonStopAfterMinutes.set_sensitive(False)
             self.buttonConfirm.set_sensitive(False)
+            self.buttonShowRestart.set_sensitive(True)
             self.buttonShowConnected.set_sensitive(False)
             self.buttonConnectedIcon.set_sensitive(False)
         self.parser.set('options', 'trackingEnabled', str(trackingEnabled))
@@ -347,9 +417,19 @@ class Main(object):
     def buttonConfirm_clicked(self, dummy):
         global confirmDisconnect
         confirmDisconnect = not confirmDisconnect
+        if confirmDisconnect:
+            self.buttonShowRestart.set_sensitive(True)
+        else:
+            self.buttonShowRestart.set_sensitive(False)
         self.parser.set('options', 'confirmDisconnect', str(confirmDisconnect))
         self.write_config()
 
+    def buttonShowRestart_clicked(self, dummy):
+        global showRestartButton
+        showRestartButton = not showRestartButton
+        self.parser.set('options', 'showRestartButton', str(showRestartButton))
+        self.write_config()
+
     def buttonShowConnected_clicked(self, dummy):
         global showConnected
         showConnected = not showConnected
@@ -385,21 +465,43 @@ class Main(object):
         global keepWifiPMOff
         keepWifiPMOff = not keepWifiPMOff
         if keepWifiPMOff:
-            self.buttonRefreshPMAfterSec.set_sensitive(True)
+            self.buttonToggleWifiPM.set_sensitive(True)
+            self.buttonWifiPMOffSec.set_sensitive(True)
+            self.buttonWifiPMOnSec.set_sensitive(True)
             system("sudo /usr/sbin/openssh-status.sh enable_wifipmoff")
         else:
-            self.buttonRefreshPMAfterSec.set_sensitive(False)
+            self.buttonToggleWifiPM.set_sensitive(False)
+            self.buttonWifiPMOffSec.set_sensitive(False)
+            self.buttonWifiPMOnSec.set_sensitive(False)
             system("sudo /usr/sbin/openssh-status.sh disable_wifipmoff")
         
-    def selectPMRefreshTime(self):
+    def buttonToggleWifiPM_clicked(self, dummy):
+        global toggleWifiPM
+        toggleWifiPM = not toggleWifiPM
+        if toggleWifiPM:
+            system("sudo /usr/sbin/openssh-status.sh enable_togglepm")
+        else:
+            system("sudo /usr/sbin/openssh-status.sh disable_togglepm")
+        
+    def selectWifiPMOffSec(self):
         selector = hildon.TouchSelector(text = True)
-        for i in self.pmRefreshList:
+        for i in self.pmOffList:
             selector.append_text(i)
         return selector
         
-    def buttonRefreshPMAfterSec_changed(self, dummy):
-        t = self.buttonRefreshPMAfterSec.get_value()
-        system("sudo /usr/sbin/openssh-status.sh set_wifipmrefresh "+t)
+    def buttonWifiPMOffSec_changed(self, dummy):
+        t = self.buttonWifiPMOffSec.get_value().split(' ')[0]
+        system("sudo /usr/sbin/openssh-status.sh set_pmoffsec "+t)
+        
+    def selectWifiPMOnSec(self):
+        selector = hildon.TouchSelector(text = True)
+        for i in self.pmOnList:
+            selector.append_text(i)
+        return selector
+        
+    def buttonWifiPMOnSec_changed(self, dummy):
+        t = self.buttonWifiPMOnSec.get_value().split(' ')[0]
+        system("sudo /usr/sbin/openssh-status.sh set_pmonsec "+t)
         
     def write_config(self):
         configFile = open('/home/user/.openssh-status-widget', 'r+')
