@@ -22,6 +22,7 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         self.stopAfterMinutes = 5
         self.keepWifiPMOff = True
         self.confirmDisconnect = True
+        self.showRestartButton = False
         self.showConnected = True
         connectedIcon = "symbol"
         self.skipSystemCheck = False
@@ -44,6 +45,8 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                 self.keepWifiPMOff = parser.getboolean('options', 'keepWifiPMOff')
             if parser.has_option('options', 'confirmDisconnect'):
                 self.confirmDisconnect = parser.getboolean('options', 'confirmDisconnect')
+            if parser.has_option('options', 'showRestartButton'):
+                self.showRestartButton = parser.getboolean('options', 'showRestartButton')
             if parser.has_option('options', 'showConnected'):
                 self.showConnected = parser.getboolean('options', 'showConnected')
             if parser.has_option('options', 'connectedIcon'):
@@ -395,10 +398,14 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         self.sshUsers()
         if (self.sshRunning or self.sshConnected) and (self.confirmDisconnect or not self.trackingEnabled):
             self.dialog = gtk.Dialog("OpenSSH status", None)
-            text = gtk.Label("Disconnect and stop server?\n")
-            text.set_line_wrap(True)
-            buttonDisconnectStop = hildon.Button(gtk.HILDON_SIZE_THUMB_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-            buttonDisconnectStop.set_label('Disconnect and stop server')
+            if not self.showRestartButton:
+                buttonDisconnectStop = hildon.Button(gtk.HILDON_SIZE_THUMB_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+                buttonDisconnectStop.set_label('Disconnect and stop server')
+            else:
+                buttonDisconnectStop = hildon.Button(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_THUMB_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+                buttonDisconnectStop.set_label('Disconnect\nStop server')
+                buttonDisconnectRestart = hildon.Button(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_THUMB_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+                buttonDisconnectRestart.set_label('  Disconnect\nRestart server')
             if not self.trackingEnabled:
                 buttonConnected = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
                 buttonConnected.set_label('Connections')
@@ -408,11 +415,16 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                     self.printInfo(str(self.sshConnected)+" connections")
             vbox2 = gtk.VBox(False,2)
             vbox2.set_border_width(8)
+            hbox = gtk.HBox(False,0)
             if not self.trackingEnabled:
                 vbox2.pack_start(buttonConnected)
                 buttonConnected.connect("clicked", self.buttonConnected_clicked)
-            vbox2.pack_start(buttonDisconnectStop)
+            hbox.pack_start(buttonDisconnectStop)
+            if self.showRestartButton:
+                hbox.pack_start(buttonDisconnectRestart)
+                buttonDisconnectRestart.connect("clicked", self.buttonDisconnectRestart_clicked)
             buttonDisconnectStop.connect("clicked", self.buttonDisconnectStop_clicked)
+            vbox2.pack_start(hbox)
             self.dialog.vbox.pack_start(vbox2)
             self.dialog.show_all()
         elif self.sshRunning or self.sshConnected:
@@ -428,6 +440,10 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         self.dialog.destroy()
         self.sshDisconnect()
     
+    def buttonDisconnectRestart_clicked(self, dummy):
+        self.dialog.destroy()
+        self.sshDisconnectRestart()
+    
     def buttonConnected_clicked(self, dummy):
         self.sshUsers()
         if self.sshConnected == 1:
@@ -464,6 +480,38 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                 self.button.set_image(self.image_off)
                 self.set_status_area_icon(None)
             
+    def sshDisconnectRestart(self):
+        self.sshStatus()
+        self.sshUsers()
+        if self.sshRunning:
+            system("sudo /usr/sbin/openssh-status.sh kill_sshd")
+            time.sleep(1)
+            self.sshStatus()
+            self.sshUsers()
+            if self.sshRunning:
+                self.printDialog("Could NOT restart OpenSSH Server!")
+            elif self.sshConnected:
+                self.printDialog("Could NOT close OpenSSH connections!")
+            else:
+                system("sudo /usr/sbin/openssh-status.sh start_sshd")
+                if self.trackingEnabled:
+                    self.button.set_text("OpenSSH Server", "0 connections")
+                self.set_status_area_icon(self.icon)
+        elif self.sshConnected:
+            system("sudo /usr/sbin/openssh-status.sh kill_sshd")
+            time.sleep(1)
+            self.sshUsers()
+            if self.sshConnected:
+                self.printDialog("Could NOT close OpenSSH connections!")
+            else:
+                system("sudo /usr/sbin/openssh-status.sh start_sshd")
+                if self.trackingEnabled:
+                    self.button.set_text("OpenSSH Server", "0 connections")
+                else:
+                    self.button.set_text("OpenSSH Server", "On")
+                self.button.set_image(self.image_on)
+                self.set_status_area_icon(self.icon)
+            
     def printDialog(self, msg):
         system("run-standalone.sh dbus-send --print-reply \
         --type=method_call --dest=org.freedesktop.Notifications \
